from unittest import TestCase
from es_translator.interpreters import Argos

class TestArgos(TestCase):
    @classmethod
    def setUpClass(self):
        self.spa2eng = Argos(source = 'spa', target = 'eng')
        self.eng2spa = Argos(source = 'eng', target = 'spa')
        self.fra2eng = Argos(source = 'fra', target = 'eng')
  
    def test_has_eng_to_spa_is_supported(self):
        self.assertTrue(self.eng2spa.is_pair_available)
    
    def test_has_spa_to_eng_is_supported(self):
        self.assertTrue(self.spa2eng.is_pair_available)
        
    def test_has_fra_to_eng_is_supported(self):
        self.assertTrue(self.fra2eng.is_pair_available)

    def test_has_ava_to_kau_isnt_supported(self): 
        with self.assertRaises(Exception):
          self.argos = Argos(source = 'ava', target = 'kau')
          self.assertFalse(self.argos.is_pair_available)

    def test_translation_from_fra_to_eng(self):
        self.assertEqual(self.fra2eng.translate('bonjour!').lower(), 'hello!')
            
    def test_translation_from_de_to_en(self):
        self.assertEqual(self.fra2eng.translate('bonjour monsieur, comment ça va ?'), 'Hello, sir, how are you?')


import click
import logging

from es_translator import EsTranslator
from tempfile import mkdtemp
# Module from the same package
from es_translator.interpreters import Apertium, Argos
from es_translator.interpreters.apertium.pairs import Pairs
from es_translator.logger import add_syslog_handler, add_stdout_handler


def validate_loglevel(ctx, param, value):
    try:
        if isinstance(value, str):
            return getattr(logging, value)
        return int(value)
    except (AttributeError, ValueError):
        raise click.BadParameter('must be a valid log level (CRITICAL, ERROR, WARNING, INFO, DEBUG or NOTSET)')


def validate_progressbar(ctx, param, value):
    # If no value given, we activate the progress bar only when the
    # stdout_loglevel value is higher than INFO (20)
    return value if value is not None else ctx.params['stdout_loglevel'] > 20


def validate_interpreter(ctx, param, value):
    interpreters = ( Apertium, Argos, )
    for interpreter in interpreters:
        if value.upper() == interpreter.name.upper():
            return interpreter
    names = (interpreter.name for interpreter in interpreters)
    raise click.BadParameter('must be a valid interpreter name (%s)' % ', '.join(names))


@click.command()
@click.option('--url', help='Elastichsearch URL', required=True)
@click.option('--index', help='Elastichsearch Index', required=True)
@click.option('--interpreter', help='Interpreter to use to perform the translation', default='APERTIUM', callback=validate_interpreter)
@click.option('--source-language', help='Source language to translate from', required=True, default=None)
@click.option('--target-language', help='Target language to translate to', required=True, default=None)
@click.option('--intermediary-language', help='An intermediary language to use when no translation is available between the source and the target. If none is provided this will be calculated automatically.')
@click.option('--source-field', help='Document field to translate', default="content")
@click.option('--target-field', help='Document field where the translations are stored', default="content_translated")
@click.option('--query-string', help='Search query string to filter result')
@click.option('--data-dir', help='Path to the directory where to language model will be downloaded', type=click.Path(exists=True, dir_okay=True, writable=True, readable=True), default=mkdtemp())
@click.option('--scan-scroll', help='Scroll duration (set to higher value if you\'re processing a lot of documents)', default="5m")
@click.option('--dry-run', help='Don\'t save anything in Elasticsearch', is_flag=True, default=False)
@click.option('--pool-size', help='Number of parallel processes to start', default=1)
@click.option('--pool-timeout', help='Timeout to add a translation', default=60 * 30)
@click.option('--throttle', help='Throttle between each translation (in ms)', default=0)
@click.option('--syslog-address', help='Syslog address', default='localhost')
@click.option('--syslog-port', help='Syslog port', default=514)
@click.option('--syslog-facility', help='Syslog facility', default='local7')
@click.option('--stdout-loglevel', help='Change the default log level for stdout error handler', default='ERROR',
              callback=validate_loglevel)
@click.option('--progressbar/--no-progressbar', help='Display a progressbar', default=None,
            callback=validate_progressbar)
def translate(syslog_address, syslog_port, syslog_facility, **options):
    # Configure Syslog handler
    add_syslog_handler(syslog_address, syslog_port, syslog_facility)
    add_stdout_handler(options['stdout_loglevel'])
    # We pass all options to EsTranslator then we start the translation
    # from Elasticsearch. This will download required pairs if needed.
    EsTranslator(options).start()


@click.command()
@click.option('--data-dir', help='Path to the directory where to language model will be downloaded', type=click.Path(exists=True, dir_okay=True, writable=True, readable=True), default=mkdtemp())
@click.option('--local', help='List pairs available locally', is_flag=True, default=False)
@click.option('--syslog-address', help='Syslog address', default='localhost')
@click.option('--syslog-port', help='Syslog port', default=514)
@click.option('--syslog-facility', help='Syslog facility', default='local7')
@click.option('--stdout-loglevel', help='Change the default log level for stdout error handler', default='ERROR',
              callback=validate_loglevel)
def pairs(data_dir, local, syslog_address, syslog_port, syslog_facility, **options):
    # Configure Syslog handler
    add_syslog_handler(syslog_address, syslog_port, syslog_facility)
    add_stdout_handler(options['stdout_loglevel'])
    # Only the data-dir is needed to construct the Apertium instance, then
    # we just need to print the pair
    Pairs(data_dir, local).print_pairs()

if __name__ == '__main__':
    translate()



from click import BadParameter
from unittest import TestCase
from es_translator.cli import validate_interpreter
from es_translator.interpreters import Apertium, Argos

class TestCli(TestCase):

    def test_it_validates_apertium_interpreter_in_uppercase(self):
        validated = validate_interpreter(None, None, 'APERTIUM')
        self.assertEqual(validated, Apertium)
        
    def test_it_validates_apertium_interpreter_in_lowercase(self):
        validated = validate_interpreter(None, None, 'apertium')
        self.assertEqual(validated, Apertium)
        
    def test_it_validates_apertium_interpreter_in_titlecase(self):
        validated = validate_interpreter(None, None, 'Apertium')
        self.assertEqual(validated, Apertium)
            
    def test_it_validates_argos_interpreter_in_uppercase(self):
        validated = validate_interpreter(None, None, 'ARGOS')
        self.assertEqual(validated, Argos)
        
    def test_it_validates_argos_interpreter_in_lowercase(self):
        validated = validate_interpreter(None, None, 'argos')
        self.assertEqual(validated, Argos)
        
    def test_it_validates_argos_interpreter_in_titlecase(self):
        validated = validate_interpreter(None, None, 'Argos')
        self.assertEqual(validated, Argos)
        
    def test_it_doesnt_validate_foo_interpreter(self):
        with self.assertRaises(BadParameter):
            validate_interpreter(None, None, 'foo')
    


import os
from os.path import dirname
from unittest import TestCase
from unittest.mock import patch
from tempfile import mkdtemp
from es_translator.interpreters.apertium import Apertium
from es_translator.interpreters.apertium.pairs import Pairs


here = lambda: os.path.dirname(os.path.abspath(__file__))
root = lambda x: os.path.abspath(os.path.join(here(), '../../../', x))
# Use the .cache dir if it exists, or use a temporary dir
pack_dir = root('.cache/APERTIUM') if os.path.isdir(root('.cache')) else mkdtemp()

class TestPair(TestCase):
    @classmethod
    def setUpClass(self):
        self.por2fra = Apertium(source = 'por', target = 'fra', intermediary = None, pack_dir = pack_dir)

    def test_por2fra_pair_is_avalaible_locally(self):
        pairs = Pairs(data_dir = pack_dir, local = True)
        self.assertIn('por-fra', pairs.local_pairs)

    def test_por2fra_pair_is_avalaible_remotly(self):
        pairs = Pairs(data_dir = pack_dir, local = True)
        self.assertIn('por-fra', pairs.remote_pairs)

    def test_por2fra_pair_isnt_avalaible_locally(self):
        pairs = Pairs(data_dir = mkdtemp(),local = True)
        self.assertNotIn('por-fra', pairs.local_pairs)

    def test_jap2cos_pair_isnt_avalaible_remotly(self):
        pairs = Pairs(data_dir = pack_dir,local = True)
        self.assertNotIn('jap-cos', pairs.local_pairs)

    @patch('builtins.print')
    def test_print_only_local_pairs(self, mock_print):
        pairs = Pairs(data_dir = pack_dir, local = True)
        pairs.print_pairs()
        mock_print.assert_called_with('\n'.join(pairs.local_pairs))

    @patch('builtins.print')
    def test_print_only_remote_pairs(self, mock_print):
        pairs = Pairs(data_dir = pack_dir, local = False)
        pairs.print_pairs()
        mock_print.assert_called_with('\n'.join(pairs.remote_pairs))



import sys
from coloredlogs import StandardErrorHandler
from contextlib import contextmanager
from elasticsearch import Elasticsearch
from elasticsearch_dsl import Search
from multiprocessing import Pool, JoinableQueue
from queue import Full
from os import path
from rich.progress import Progress
from time import sleep
# Module from the same package
from es_translator.es import TranslatedHit
from es_translator.logger import logger


def translation_worker(queue):
    while True:
        try:
            es_translator, hit, index, throttle = queue.get(True)
            logger.info('Translating doc %s (%s)' % (index, hit.meta.id))
            translated_hit = TranslatedHit(hit, es_translator.source_field, es_translator.target_field)
            translated_hit.add_translation(es_translator.interpreter)
            logger.info('Translated doc %s (%s)' % (index, hit.meta.id))
            # Skip on dry run
            if not es_translator.dry_run:
                # Create a new  client to
                client = Elasticsearch(es_translator.url)
                translated_hit.save(client)
                logger.info('Saved translation for doc %s (%s)' % (index, hit.meta.id))
            queue.task_done()
            sleep(throttle / 1000)
        except Exception as error:
            logger.warning('Unable to translate doc %s (%s)' % (index, hit.meta.id))
            logger.warning(error)
            queue.task_done()


class EsTranslator:
    def __init__(self, options):
        self.url = options['url']
        self.index = options['index']
        self.source_language = options['source_language']
        self.target_language = options['target_language']
        self.intermediary_language = options['intermediary_language']
        self.source_field = options['source_field']
        self.target_field = options['target_field']
        self.query_string = options['query_string']
        self.data_dir = options['data_dir']
        self.scan_scroll = options['scan_scroll']
        self.dry_run = options['dry_run']
        self.pool_size = options['pool_size']
        self.pool_timeout = options['pool_timeout']
        self.throttle = options['throttle']
        self.progressbar = options['progressbar']
        # The "interpreter" option is a class
        self.Interpreter = options['interpreter']

    @property
    def no_progressbar(self):
        return not self.progressbar

    def start(self):
        with self.print_done('Instantiating %s interpreter' % self.Interpreter.name):
            self.interpreter = self.init_interpreter()

        total = self.search().count()
        desc = 'Translating %s document(s)' % total

        with self.print_done(desc):
            # Add missing field and change the scroll duration
            search = self.search()
            search = search.source([self.source_field, self.target_field, '_routing'])
            search = search.params(scroll=self.scan_scroll)
            # Create a queue that is able to translate documents in parallel
            translation_queue = JoinableQueue(self.pool_size)
            # We create a pool
            with Pool(self.pool_size, translation_worker, (translation_queue,)):
                with Progress(disable=self.no_progressbar, transient=True) as progress:   
                    documents = search.scan()         
                    task = progress.add_task(desc, total=total)
                    for index, hit in enumerate(documents):
                        translation_queue.put((self, hit, index, self.throttle), True, self.pool_timeout)
                        progress.advance(task)
                    translation_queue.join()
                    

    def search(self):
        es_client = Elasticsearch(self.url)
        search = Search(index=self.index, using=es_client)
        # Add query_string to the search
        if self.query_string:
            search = search.query("query_string", query=self.query_string)
        return search

    def init_interpreter(self):
        pack_dir = path.join(self.data_dir, 'packs', self.Interpreter.name)
        return self.Interpreter(self.source_language, self.target_language, self.intermediary_language, pack_dir)

    def print_flush(self, str):
        sys.stdout.write('\r{0}'.format(str))
        sys.stdout.flush()

    @property
    def stdout_loglevel(self):
        try:
            handler = next(h for h in logger.handlers if isinstance(h, StandardErrorHandler))
            return getattr(handler, 'level', 0)
        except StopIteration:
            return 0

    @contextmanager
    def print_done(self, str, quiet = False):
        logger.info(str)
        # Avoid conflicting with a high log level
        if self.stdout_loglevel > 20:
            str = '\r%s...' % str
            self.print_flush(str)
            try:
                yield
                print('{0} \033[92mdone\033[0m'.format(str))
            except Full:
                logger.error('Timeout reached (%ss).' % self.pool_timeout)
                print('{0} \033[91merror\033[0m'.format(str))
            except Exception as error:
                logger.error(error, exc_info=True)
                print('{0} \033[91merror\033[0m'.format(str))
                if not quiet: raise error
                sys.exit(1)
        else:
            yield



import os

from unittest import TestCase
from tempfile import mkdtemp
from es_translator.interpreters.apertium import Apertium


here = lambda: os.path.dirname(os.path.abspath(__file__))
root = lambda x: os.path.abspath(os.path.join(here(), '../../../', x))
# Use the .cache dir if it exists, or use a temporary dir
pack_dir = root('.cache/APERTIUM') if os.path.isdir(root('.cache')) else mkdtemp()

class TestApertium(TestCase):
    @classmethod
    def setUpClass(self):
        self.eng2spa = Apertium(source = 'eng', target = 'spa', intermediary = None, pack_dir = pack_dir)
        self.spa2eng = Apertium(source = 'spa', target = 'eng', intermediary = None, pack_dir = pack_dir)
        self.por2eng = Apertium(source = 'por', target = 'eng', intermediary = 'cat', pack_dir = pack_dir)
        self.por2fra = Apertium(source = 'por', target = 'fra', intermediary = None, pack_dir = pack_dir)

    def test_pair_directory_is_created(self):
        self.assertIn('apertium-%s' % self.eng2spa.pair_package, os.listdir(pack_dir))
        self.assertIn('apertium-%s' % self.spa2eng.pair_package, os.listdir(pack_dir))

    def test_intermediary_pair_directory_is_created(self):
        self.assertIn('apertium-%s' % self.por2eng.intermediary_source_pair_package, os.listdir(pack_dir))

    def test_pair_packages_list(self):
        self.assertTrue(all(len(p.split('-')) == 2 for p in self.eng2spa.remote_pairs))

    def test_source_properties(self):
        self.assertEqual(self.eng2spa.source, 'eng')
        self.assertEqual(self.eng2spa.source_alpha_2, 'en')
        self.assertEqual(self.eng2spa.source_alpha_3, 'eng')

    def test_target_properties(self):
        self.assertEqual(self.eng2spa.target, 'spa')
        self.assertEqual(self.eng2spa.target_alpha_2, 'es')
        self.assertEqual(self.eng2spa.target_alpha_3, 'spa')

    def test_source_names(self):
        self.assertEqual(self.eng2spa.source_name, 'English')

    def test_target_names(self):
        self.assertEqual(self.eng2spa.target_name, 'Spanish')

    def test_translate_simple_word(self):
        self.assertEqual(self.eng2spa.translate('hello').strip(), 'hola')
        self.assertEqual(self.eng2spa.translate('hello sir').strip(), 'hola Señor')
        self.assertEqual(self.spa2eng.translate('hola').strip(), 'hello')
        self.assertEqual(self.por2eng.translate('obrigado').strip(), 'thank you')
        self.assertEqual(self.por2fra.translate('obrigado').strip(), 'merci')



import sys

from setuptools import setup, find_packages

py_version = sys.version_info[:2]
if py_version < (3, 6):
    raise Exception("es-translator requires Python >= 3.6.")

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(name='es-translator',
      version='1.3.3',
      packages=find_packages(),
      description="A lazy yet bulletproof machine translation tool for Elastichsearch.",
      long_description=long_description,
      long_description_content_type="text/markdown",
      url="https://github.com/icij/es-translator",
      test_suite='nose.collector',
      tests_require=['nose', 'responses'],
      include_package_data=True,
      keywords=['datashare', 'api', 'text-mining', 'elasticsearch', 'apertium', 'translation'],
      classifiers=[
          "Programming Language :: Python :: 3",
          "Programming Language :: Python :: 3.6",
          "Programming Language :: Python :: 3.7",
          "Programming Language :: Python :: 3.8",
          "Intended Audience :: Developers",
          "License :: OSI Approved :: GNU Affero General Public License v3",
          "Operating System :: OS Independent",
          "Topic :: Scientific/Engineering :: Information Analysis",
      ],
      install_requires=[
        'click==8.1',
        'elasticsearch>=7.10,<7.11',
        'elasticsearch-dsl>=7.4.0,<8.0.0',
        'sh==1.14.2',
        'pycountry==22.3.5',
        'deb-pkg-tools==5.2',
        'nose==1.3.7',
        'urllib3==1.26.9',
        'rich==12.4.1',
        'argostranslate==1.6.1'
      ],
      python_requires='>=3.6',
      entry_points='''
        [console_scripts]
        es-translator=es_translator.cli:translate
        es-translator-pairs=es_translator.cli:pairs
    ''')



class TranslatedHit():
    def __init__(self, hit, source_field = 'content', target_field = 'content_translated'):
        self.hit = hit
        self.source_field = source_field
        self.target_field = target_field
        # Ensure the target field is an array
        self.hit[self.target_field] = self.translations

    @property
    def source_value(self):
        return self.hit.to_dict().get(self.source_field)

    @property
    def translations(self):
        return self.hit.to_dict().get(self.target_field, [])

    @property
    def id(self):
        return self.hit.meta.id

    @property
    def routing(self):
        # Some documents don't have a routing property
        return self.hit.meta.to_dict().get('routing', None)

    @property
    def index(self):
        return self.hit.meta.index

    @property
    def doc_type(self):
        return self.hit.meta.doc_type

    @property
    def body(self):
        body = dict(doc=dict())
        body['doc'][self.target_field] = self.translations
        return body

    def save(self, client):
        client.update(index = self.index, doc_type = self.doc_type, id = self.id, routing = self.routing, body = self.body)

    def add_translation(self, interpreter):
        if not self.is_translated(interpreter.source_name, interpreter.target_name):
            self.hit[self.target_field].append(dict(
                translator = interpreter.name,
                source_language = interpreter.source_name.upper(),
                target_language = interpreter.target_name.upper(),
                content = interpreter.translate(self.source_value)))

    def is_translated(self, source_language, target_language):
        same_languages = lambda t: t['source_language'] == source_language.upper() and t['target_language'] == target_language.upper()
        return any(t for t in self.translations if same_languages(t))



import os
from os.path import isfile, isdir

from unittest import TestCase
from tempfile import mkdtemp
from es_translator.interpreters.apertium.repository import ApertiumRepository


here = lambda: os.path.dirname(os.path.abspath(__file__))
root = lambda x: os.path.abspath(os.path.join(here(), '../../../', x))
# Use the .cache dir if it exists, or use a temporary dir
cache_dir = root('.cache/APERTIUM') if os.path.isdir(root('.cache')) else mkdtemp()

class TestApertium(TestCase):
    @classmethod
    def setUpClass(self):
        self.apr = ApertiumRepository(cache_dir = cache_dir)

    def test_has_packages_list(self):
        self.assertEqual(type(self.apr.packages), list)

    def test_find_apertium_package(self):
        self.assertTrue(self.apr.find_package('apertium') is not None)

    def test_find_apertium_enes_package(self):
        self.assertTrue(self.apr.find_package('apertium-eng-spa') is not None)

    def test_find_package_for_pair(self):
        self.assertTrue(self.apr.find_pair_package('eng-spa') is not None)

    def test_find_package_for_inversed_pair(self):
        self.assertTrue(self.apr.find_pair_package('spa-eng') is not None)

    def test_pair_packages(self):
        self.assertEqual(type(self.apr.pair_packages), list)

    def test_is_apertium_pair(self):
        self.assertTrue(self.apr.is_apertium_pair(dict(Package='apertium-eng-spa')))
        self.assertTrue(self.apr.is_apertium_pair(dict(Package='apertium-spa-cat')))
        self.assertFalse(self.apr.is_apertium_pair(dict(Package='nop-eng-spa')))
        self.assertFalse(self.apr.is_apertium_pair(dict(Package='eng-spa')))

    def test_download_package(self):
        package_file = self.apr.download_package('apertium-eng-spa')
        self.assertTrue(isfile(package_file))

    def test_download_pair_package_in_alpha_2(self):
        package_file = self.apr.download_pair_package('pt-fr')
        self.assertTrue(isfile(package_file))

    def test_download_pair_package_in_alpha_3(self):
        package_file = self.apr.download_pair_package('por-fra')
        self.assertTrue(isfile(package_file))

    def test_download_pair_package_inversed(self):
        package_file = self.apr.download_pair_package('por-cat')
        self.assertTrue(isfile(package_file))

    def test_extract_pair_package(self):
        package_file = self.apr.download_pair_package('eng-spa')
        package_directory = self.apr.extract_pair_package(package_file)
        self.assertTrue(isdir(package_directory))
        self.assertTrue(isdir(package_directory + '/modes'))
        self.assertTrue(isdir(package_directory + '/apertium-eng-spa'))

    def test_import_modes(self):
        self.apr.clear_modes()
        self.assertFalse(isdir(cache_dir + '/modes'))
        package_file = self.apr.download_pair_package('eng-spa')
        self.apr.extract_pair_package(package_file)
        self.apr.import_modes(clear = False)
        self.assertTrue(isdir(cache_dir + '/modes'))
        self.assertTrue(isfile(cache_dir + '/modes/eng-spa.mode'))
        self.assertTrue(isfile(cache_dir + '/modes/spa-eng.mode'))

    def test_install_pair_package_alpha3(self):
        self.apr.clear_modes()
        self.assertFalse(isdir(cache_dir + '/modes'))
        self.apr.install_pair_package('fra-cat')
        self.assertTrue(isdir(cache_dir + '/modes'))
        self.assertTrue(isfile(cache_dir + '/modes/fra-cat.mode'))

    def test_install_pair_package_reversed(self):
        self.apr.clear_modes()
        self.assertFalse(isdir(cache_dir + '/modes'))
        self.apr.install_pair_package('spa-eng')
        self.assertTrue(isdir(cache_dir + '/modes'))
        self.assertTrue(isfile(cache_dir + '/modes/eng-spa.mode'))
        self.assertTrue(isfile(cache_dir + '/modes/spa-eng.mode'))
